/*
Tecnisa - Tecnologia em Informtica

Nome: APURAC_GESMINTE

Descrio:
   Procedimento para apurao de faturamento de clientes em filiais
   que utilizam o tipo de contrato Gesto Minte.

Criao:
28/08/2000 - Roberto A. Lopes Martins

@C:\WIN_PRJ\Sql\Oracle\6203_FAISTP_ApuGesMinte.sql
raise_application_error(-20501, '' , True);
*/

CREATE OR REPLACE PROCEDURE APURAC_GESMINTE(
      P_CDFILIAPUR  IN FILIAL.CDFILIAL%TYPE,
      P_CDFILIPROC  IN FILIAL.CDFILIAL%TYPE,
      P_NRLANCTONF  IN NOTAFISCAL.NRLANCTONF%TYPE,
      P_CDCLIENTE   IN CLIENTE.CDCLIENTE%TYPE,
      P_DTINICIAL   IN DATE,
      P_DTFINAL     IN DATE,
      P_NRAPURACFAT IN NOTAFISCAL.NRAPURACFAT%TYPE,
      P_IDNFCOMPRA  IN VARCHAR)

AS
   /*Variveis contendo parmetros do faturamento*/
   V_CDEMPRATAC  EMPRESA.CDEMPRESA%TYPE;
   V_CDEMPROPER  EMPRESA.CDEMPRESA%TYPE;

   V_CDTIPOOPER   TIPOOPERAC.CDTIPOOPER%TYPE;
   /*Filial que est sendo apurada*/
   V_CDFILIAPUR  CTRLAPURFAT.CDFILIAPUR%TYPE;

   /*Srie da empresa operacional */
   V_CDSERIEOPER SERIENF.CDSERIE%TYPE;
   V_EXISTEMPR   NUMBER;

   /*Taxa de compra*/
   V_VRPCTXADM   NUMBER;

   V_CDPRPRESERV PRODUTO.CDPRODUTO%TYPE;
   V_CDPRGENEREV PRODUTO.CDPRODUTO%TYPE;
   V_NMPRESTSERV  PRODUTO.NMPRODUTO%TYPE;

   V_NRLANCTONF  NOTAFISCAL.NRLANCTONF%TYPE;
   V_NRLANCESTQ  LANCTOESTOQ.NRLANCESTQ%TYPE;

   V_NRSEQITPROD ITEMPRODUTO.NRSEQITPROD%TYPE;

   /*Variveis para gerao da nota fiscal de prestao de servios*/
   V_VRPRESTSERV NUMBER;
   V_VRSERVICO   NUMBER;
   V_VRSERVICOGD   NUMBER;
   V_VRSERVICOTU   NUMBER;
   V_VRGENEROS   NUMBER;
   V_NMPRGENEREV PRODUTO.NMPRODUTO%TYPE;


   /*Cursor auxiliar para deletar as notas fiscais*/
   CURSOR C_DELETANF IS
      SELECT CDFILIAL, NRLANCTONF
      FROM NOTAFISCAL
      WHERE CDFILIAL    = P_CDFILIPROC  AND
            NRAPURACFAT = P_NRAPURACFAT AND
            CDCLIENTE   = P_CDCLIENTE;

BEGIN
   V_CDFILIAPUR := P_CDFILIAPUR;

   /*Parmetros do faturamento:*/
   SELECT TO_NUMBER(NVL(MAX(SUBSTR(PF.DSPARAMFATFI,31,6)),0)), /*Taxa de compra*/
          MAX(SUBSTR(PF.DSPARAMFATFI,39,2)),  /*Empresa atacadista*/
          MAX(SUBSTR(PF.DSPARAMFATFI,41,2)),  /*Empresa operacional*/
          MAX(SUBSTR(PG.DSPARAMFATGE,12,10)), /*Codigo da Produto para Faturamento (prestao de servios)*/
          MAX(SUBSTR(PG.DSPARAMFATGE,73,10)), /*Cdigo do produto para Faturamento*/
          MAX(SUBSTR(PF.DSPARAMFATFI,70,2)) /*TIPO DE OPERACAO --parametros por tipo de faturamento */
   INTO V_VRPCTXADM, V_CDEMPRATAC,  V_CDEMPROPER,
        V_CDPRPRESERV, V_CDPRGENEREV, V_CDTIPOOPER
   FROM PARAMFILIAL PF, PARAMGERAL PG
   WHERE PF.CDFILIAL = V_CDFILIAPUR;

   /*Taxa de Compra*/
   V_VRPCTXADM := V_VRPCTXADM/1000;

   /*Atualizao da chave da apurao na tabela de nota fiscal, s  feito
     quando o usurio escolhe nova verificao na tela, caso contrrio, a
     atualizao  feita no programa*/
   IF P_IDNFCOMPRA = 'N' THEN
      UPDATE NOTAFISCAL
      SET  CDFILIAPFATU  = P_CDFILIAPUR,
           NRAPFATUTNF   = P_NRAPURACFAT
      WHERE (CDFILIRECE   = V_CDFILIAPUR)                      AND
            (CDEMPRESA    = V_CDEMPRATAC)                       AND
            (DTENTRSAID  BETWEEN  P_DTINICIAL AND P_DTFINAL)  AND
            (IDENTRSAIDOP = 'E')                                AND
            (IDSTATUS     <> 'C')                               AND
            ((IDSISTEMA = 'DFE') OR (IDSISTEMA IS NULL))        AND
            ((NRAPFATUTNF IS NULL) OR (NRAPFATUTNF = P_NRAPURACFAT)) AND
            (CDTIPOOPER IN (SELECT CDTIPOOPER
                            FROM TIPOOPERAC
                            WHERE IDENTRSAIDOP = 'E' AND
                                  IDUTILOPERFAMAN = 'S'));
   END IF;

   IF (V_CDTIPOOPER IS NULL) OR (V_CDTIPOOPER = '  ') THEN
      V_CDTIPOOPER := '01';
   END IF;

   /*Busca da srie da empresa operacional na tabela de filial UTILIZADA PARA GERAR A SERIE DE SAIDA*/
   SELECT MAX(CDSERIE), COUNT(*)
   INTO V_CDSERIEOPER, V_EXISTEMPR
   FROM FILIAL
   WHERE CDFILIAL  = V_CDFILIAPUR AND
         CDEMPRESA = V_CDEMPROPER;

   /*Busca da srie da empresa operacional na tabela de empresas relacionadas*/
   IF (V_CDSERIEOPER IS NULL) AND (V_EXISTEMPR = 0) THEN
      SELECT MAX(CDSERIE)
      INTO V_CDSERIEOPER
      FROM EMPRCNTRFIL
      WHERE CDFILIAL  = V_CDFILIAPUR AND
            CDEMPRESA = V_CDEMPROPER;
   END IF;

   /*Apaga todas as notas fiscais apuradas anteriormente com mesmo
     numero de apuracao*/
   FOR C_NOTAFISCAL IN C_DELETANF LOOP
      DELETE IMPOSINFUSU
      WHERE CDFILIAL   = C_NOTAFISCAL.CDFILIAL AND
            NRLANCTONF = C_NOTAFISCAL.NRLANCTONF;

      DELETA_NF(C_NOTAFISCAL.CDFILIAL, C_NOTAFISCAL.NRLANCTONF);
   END LOOP;

   V_NRLANCTONF := P_NRLANCTONF;

   SELECT MAX(NMPRODUTO)
   INTO V_NMPRGENEREV
   FROM PRODUTO
   WHERE CDPRODUTO  =  V_CDPRGENEREV;

   APURAC_GESDIRETA(P_CDFILIPROC, P_CDCLIENTE, P_DTINICIAL, P_DTFINAL, P_NRAPURACFAT, 'N');

   /*Totalizao dos servios apurados*/
   SELECT NVL(SUM(VRPRECUNITAPGD * NREFETIVOSAPGD),0)
   INTO V_VRSERVICOGD
   FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD  = P_CDFILIAPUR  AND
         NRAPURACGD    = P_NRAPURACFAT AND
         CDCLIENTEAPGD = P_CDCLIENTE;

   SELECT NVL(SUM(VRPRECUNITAPTU * NREFETIVOSAPTU),0)
   INTO V_VRSERVICOTU
   FROM TRAB_APURAC_TU
   WHERE CDFILIALAPTU  = P_CDFILIAPUR  AND
         NRAPURACTU    = P_NRAPURACFAT AND
         CDCLIENTEAPTU = P_CDCLIENTE;

   V_VRSERVICO := V_VRSERVICOGD + V_VRSERVICOTU;
   /*Totalizao dos documentos de compra*/
   SELECT NVL(SUM(VRNOTAFISC),0)
   INTO V_VRGENEROS
   FROM NOTAFISCAL
   WHERE CDFILIAPFATU    = P_CDFILIAPUR AND
         NRAPFATUTNF = P_NRAPURACFAT;

   /*Gerao da nota fiscal de prestao de servios
    Valor de Faturamento - (valor da entradas + taxa de compra)*/
   V_VRPRESTSERV :=  (V_VRSERVICO - (V_VRGENEROS + ((V_VRGENEROS *V_VRPCTXADM)/100)));

   /*Chamada da apurao Gesto Direta para totalizao dos servios no perodo*/
   IF V_VRPRESTSERV > 0 THEN

      GERA_NOTAFISCAL_CORPO(P_CDFILIPROC,     V_CDFILIAPUR,  V_CDEMPROPER,
                            V_CDTIPOOPER,     V_CDSERIEOPER, P_CDCLIENTE,
                            P_DTFINAL,        'FAI',        V_NRLANCTONF,
                            V_NRLANCESTQ, 'N',          P_NRAPURACFAT,
                            V_NRLANCTONF,  NULL,         NULL,
                            NULL,          NULL,         NULL,
                            NULL,          NULL,         NULL,
                            NULL,          NULL,         NULL,
                            NULL,          NULL,         NULL,
                            NULL,          P_DTFINAL,    NULL);



      GERA_NOTAFISCAL_PRODUTO(P_CDFILIPROC, V_NRLANCTONF, V_CDPRGENEREV,
                              V_CDPRGENEREV, V_NMPRGENEREV, 1, V_VRPRESTSERV, 0,
                              NULL, NULL, NULL, NULL,
                              NULL, NULL);

      GERA_NOTAFISCAL_FECHAMENTO(P_CDFILIPROC, V_NRLANCTONF);
   END IF;

   /*Apaga registros na tabela de trabalho*/
   DELETE FROM TRAB_APURAC_GD
   WHERE CDFILIALAPGD  = P_CDFILIAPUR  AND
         NRAPURACGD    = P_NRAPURACFAT AND
         CDCLIENTEAPGD = P_CDCLIENTE ;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('APURAC_GESMINTE','5.01.003'); END;
/

